Public Class Interpreter

    ' Called when the different positions changes
    Public Event CommandLocationChanged()
    Public Event MemoryLocationChanged()

    ' The text box which contains all the commands
    Public Commands As TextBox

    ' The array with all the memory
    Public Memory As Array

    ' The current command and memory index
    Public CommandIndex As Int32
    Public MemoryIndex As Int32
    Public InputIndex As Int32

    ' Input and output
    Public InputBox As TextBox
    Public OutputBox As TextBox

    ' Whether or not the user wants to stop the loop
    Private bStop As Boolean

    ' Called when the user wants to run the code
    Public Sub Run(Optional ByVal Interval As Int32 = 10, Optional ByVal Lenght As Int32 = -1, Optional ByVal Direction As Int32 = 1)

        ' Initialize variables
        bStop = False

        ' Run till we've been ordered to stop
        Do Until bStop Or (Lenght = 0) Or ((CommandIndex >= Commands.TextLength) _
         And (Direction > 0)) Or ((CommandIndex < 0) And (Direction < 0))

            ' Move to the next command
            If Direction < 0 Then
                CommandIndex += Direction
            End If

            ' Execute the current command
            ExecuteCommand(Commands.Text.Substring(CommandIndex, 1), Direction)

            If Direction > 0 Then
                CommandIndex += Direction
            End If

            ' The location of the command has changed
            RaiseEvent CommandLocationChanged()

            ' Decrease the lenght-variable
            If Lenght > -1 Then
                Lenght -= 1
            End If

            ' Wait the given amount of time
            System.Threading.Thread.Sleep(Interval)

            ' Execute events
            Application.DoEvents()

        Loop

    End Sub

    Public Sub ExecuteCommand(ByVal sCommand As String, ByVal Direction As Int32)

        Dim Value As Int32

        ' Execute depending on the command
        Select Case sCommand
            Case ">"
                MemoryIndex += Direction
            Case "<"
                MemoryIndex -= Direction
            Case "+"
                Memory.SetValue(Memory.GetValue(MemoryIndex) + Direction, MemoryIndex)
            Case "-"
                Memory.SetValue(Memory.GetValue(MemoryIndex) - Direction, MemoryIndex)
            Case "."

                ' Here we'll add or remove a character from the output stream, depending on the direction
                If Direction > 0 Then
                    OutputBox.Text = OutputBox.Text & ChrW(Memory.GetValue(MemoryIndex))
                Else
                    OutputBox.Text = OutputBox.Text.Substring(0, OutputBox.Text.Length - 1)
                End If

            Case ","

                ' If we are to remove a input, we'll just assume the original value was zero
                If Direction > 0 Then
                    Value = Asc(InputBox.Text.Substring(InputIndex, 1))
                End If

                ' Read a character from the input
                Memory.SetValue(Value, MemoryIndex)

                ' Move to the next character
                InputIndex += Direction

            Case "[" ' Jump forward to the next ] if zero

                If Memory.GetValue(MemoryIndex) = 0 Then
                    CommandIndex = FindIndex(Commands, CommandIndex, Commands.TextLength, "]", 1) + Direction
                End If

            Case "]" ' Jump backwards if nonzero

                If Memory.GetValue(MemoryIndex) <> 0 Then
                    CommandIndex = FindIndex(Commands, CommandIndex, 1, "[", -1) - Direction
                End If

        End Select

        ' We must most certainly update the memory list
        RaiseEvent MemoryLocationChanged()

    End Sub

    Private Function FindIndex(ByVal refBox As TextBox, ByVal Start As Int32, ByVal EndIndex As Int32, ByVal sChar As String, ByVal Direction As Int32) As Int32

        Dim Tell As Long

        ' Search for the given character
        For Tell = Start To EndIndex Step Direction

            ' See whether or not this character contains the given string
            If refBox.Text.Substring(Tell, 1) = sChar Then

                ' Return the given index
                FindIndex = Tell

                ' We're done searching
                Exit Function

            End If

        Next

    End Function

    Public Sub Break()

        ' Stops the execution of the program
        bStop = True

    End Sub

End Class
