Public Class frmMain

    ' The code-dialog
    Private Code As frmCode

    ' Output- and Input-dialogs
    Private OutputForm As frmCode
    Private InputForm As frmCode

    ' The memory-content 
    Private Memory As frmMemory

    ' The array to contain all the memory
    Private aMemory() As Int32

    ' The module that interpretates the code
    Public WithEvents Interpreter As Interpreter

    Private Sub frmMain_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load

        ' Just open a empty project
        OpenProject("")

    End Sub

    Public Sub OpenProject(ByVal sFile As String)

        Dim sCode As String = "", UTF As New System.Text.UTF7Encoding()

        ' Initialize the memory
        ReDim aMemory(1024)

        ' Open the file, if it exitsts (otherwise, just use a empty string
        If System.IO.File.Exists(sFile) Then
            sCode = UTF.GetString(System.IO.File.ReadAllBytes(sFile))
        End If

        ' Show the different forms
        ShowForm(Code, New frmCode, "Code editor").txtCode.Text = sCode
        ShowForm(Memory, New frmMemory, "Memory").Memory = aMemory
        ShowForm(OutputForm, New frmCode, "Output").txtCode.Enabled = False
        ShowForm(InputForm, New frmCode, "Input")

        ' Set the default-positions
        cmdResize_Click(Me, New System.EventArgs)

        ' Initialize the interpreter
        Interpreter = New Interpreter
        Interpreter.CommandIndex = 0
        Interpreter.MemoryIndex = 0
        Interpreter.Commands = Code.txtCode
        Interpreter.OutputBox = OutputForm.txtCode
        Interpreter.Memory = aMemory

    End Sub

    Public Function ShowForm(ByRef codeForm As Object, ByVal newForm As Object, ByVal Caption As String) As Object

        ' Close the form, if it exitst
        If Not (codeForm Is Nothing) Then
            codeForm.close()
        End If

        ' Show a code-form
        codeForm = newForm

        ' Set the caption
        codeForm.Text = Caption

        ' Set the MDI-parent to use
        codeForm.MdiParent = Me

        ' Show it
        codeForm.Show()

        ' Return the given form
        Return codeForm

    End Function

    Private Sub Interpreter_CommandLocationChanged() Handles Interpreter.CommandLocationChanged

        ' Change the selected item in the code-box
        If Interpreter.CommandIndex < 0 Then
            Code.txtCode.Select(0, 0)
        Else
            Code.txtCode.Select(Interpreter.CommandIndex, 1)
        End If

        ' Update the statusbar
        lblCommandIndex.Text = "Command: " & Interpreter.CommandIndex

    End Sub

    Private Sub Interpreter_MemoryLocationChanged() Handles Interpreter.MemoryLocationChanged

        Try

            ' Select the correct item in the list view
            Memory.lstMemory.Items(Interpreter.MemoryIndex).Selected = True

            ' Update the current item
            Memory.lstMemory.RedrawItems(Interpreter.MemoryIndex, Interpreter.MemoryIndex, False)

            ' Update the statusbar
            lblMemoryIndex.Text = "Memory: " & Interpreter.MemoryIndex

        Catch ex As System.Exception

            ' Inform about the exception
            MessageBox.Show("Out of memory bounds.", "Error")

            ' Stop execution
            cmdReset_Click(Me, Nothing)

        End Try

    End Sub

    Private Sub cmdPlay_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdPlay.Click

        ' Select the textbox
        Code.Select()

        ' Start executiom
        Interpreter.Run()

    End Sub

    Private Sub cmdForward_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles cmdForward.Click

        ' Make sure the command index is valid
        If Interpreter.CommandIndex < 0 Then
            Interpreter.CommandIndex = 0
        End If

        ' Execute and move forward one time
        Interpreter.Run(0, 1)

    End Sub

    Private Sub cmdStop_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdStop.Click

        ' Stop all
        Interpreter.Break()

    End Sub

    Private Sub cmdClear_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdClear.Click

        ' Simply open a new project
        OpenProject("")

    End Sub

    Private Sub cmdReset_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdReset.Click

        ' Reset the locations
        Interpreter.MemoryIndex = 0
        Interpreter.CommandIndex = 0

        ' Reset the memory
        ReDim aMemory(1024)

        ' Reinitialize the memory
        Memory.Memory = aMemory
        Interpreter.Memory = aMemory

        ' Initiate the events
        Interpreter_CommandLocationChanged()
        Interpreter_MemoryLocationChanged()

        ' Redraw the list view
        Memory.lstMemory.Invalidate()

    End Sub

    Private Sub cmdResize_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdResize.Click

        ' Resize and relocate the different forms
        OutputForm.Left = 0
        OutputForm.Height = 120
        InputForm.Left = 0
        InputForm.Height = 120
        OutputForm.Width = Me.ClientSize.Width - Memory.Width - 5
        InputForm.Width = OutputForm.Width
        Code.Width = OutputForm.Width
        Code.Height = Me.ClientSize.Height - OutputForm.Height _
         - InputForm.Height - statusBar.Height - toolBar.Height - 5
        InputForm.Top = Code.Height
        OutputForm.Top = InputForm.Top + InputForm.Height
        Memory.Top = 0
        Memory.Height = Me.ClientSize.Height - statusBar.Height - toolBar.Height - 5
        Memory.Left = Code.Width

    End Sub

    Private Sub cmdOpen_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdOpen.Click

        ' Firstly, show the dialog
        openFile.ShowDialog()

        ' See if the user has selected a file
        If openFile.FileName.Length <> 0 Then
            ' Open the project
            OpenProject(openFile.FileName)
        End If

    End Sub

    Private Sub cmdSave_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdSave.Click

        ' Firstly, show the dialog
        saveFile.ShowDialog()

        ' See if the user has selected a file
        If saveFile.FileName.Length <> 0 Then
            ' Save the project
            System.IO.File.WriteAllText(saveFile.FileName, Code.txtCode.Text)
        End If

    End Sub

    Private Sub cmdBack_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdBack.Click

        ' Go a step backwards
        If Interpreter.CommandIndex > 0 Then
            Interpreter.Run(0, 1, -1)
        End If

    End Sub

End Class
